providers = [
    "https://relay.flashbots.net",
    "https://builder0x69.io",
    "https://eth-builder.com",
    "https://rpc.beaverbuild.org",
    "https://rsync-builder.xyz",
    "https://api.edennetwork.io/v1/bundle",
    "https://rpc.titanbuilder.xyz",
    "https://rpc.penguinbuild.org",
    "https://rpc.nfactorial.xyz",
    "https://builder.gmbit.co/rpc",
    "https://rpc.jetbldr.xyz",
    "https://api.securerpc.com/v1",
    "https://BuildAI.net",
    "https://rpc.lokibuilder.xyz",
    "https://rpc.lightspeedbuilder.info",
    "https://rpc.payload.de",
    "https://rpc.f1b.io",
]